"""API client modules for fishinfo data.

This module provides API clients for querying telemetry data
and bulk importing legacy data (superadmin).

Note: Ingestion is handled by the ingress-edge SDK, not the fishinfo SDK.
"""

from fishinfo.api.import_api import ImportApi
from fishinfo.api.telemetry import TelemetryApi

__all__ = [
    "TelemetryApi",
    "ImportApi",
]
