"""Fishinfo SDK models.

Query-only models for telemetry data access. This SDK does NOT handle ingestion.
Ingestion is handled by the ingress-edge SDK.
"""

from fishinfo.models.import_job import (
    BulkImportResponse,
    BulkTelemetryRecord,
    CreateImportJobResponse,
    ImportFormat,
    ImportJob,
    ImportJobListResponse,
    ImportJobPart,
    ImportJobStatus,
    ImportSchemaResponse,
)
from fishinfo.models.query import (
    AggregatedDataPoint,
    AggregatedTelemetry,
    AggregationType,
    LatestTelemetryResponse,
    OrderDirection,
    PaginationMeta,
    TelemetryResponse,
)
from fishinfo.models.telemetry import DataType, TelemetryPoint

__all__ = [
    # Telemetry models
    "DataType",
    "TelemetryPoint",
    # Query response models
    "TelemetryResponse",
    "LatestTelemetryResponse",
    "PaginationMeta",
    "OrderDirection",
    "AggregationType",
    "AggregatedDataPoint",
    "AggregatedTelemetry",
    # Import models
    "ImportJobStatus",
    "ImportFormat",
    "BulkTelemetryRecord",
    "ImportJob",
    "ImportJobPart",
    "CreateImportJobResponse",
    "ImportJobListResponse",
    "BulkImportResponse",
    "ImportSchemaResponse",
]
